// date.cpp
// by David Lambert - 2013-01-30

#include <iostream>
#include "date.h"
using namespace std;

Date::Date()
{
   Set(1,1,2000);
   dateFormat = 'D';
}

Date::Date(int m, int d, int y)
{
   if (!Set(m,d,y))  //Set does error-checking
      {   
         month = 1;
         day = 1;
         year = 2000;
      }
   dateFormat = 'D';
}

void Date::Show() const
{
   string monthName[12] = { "Jan", "Feb", "Mar", "Apr", "May", "June",
      "July", "Aug", "Sept", "Oct", "Nov", "Dec" };
   int julian = 0;
   
   if (dateFormat == 'D')  //default
      { cout << month << "/" << day << "/" << year << endl; }
   else if (dateFormat == 'T')   //two-digit
      { 
         if (month < 10)      //leading zero
            cout << 0 << month << "/";
         else
            cout << month << "/";

         if (day < 10)     //leading zero
            cout << 0 << day << "/";
         else
            cout << day << "/";
         
         if ((year % 100) < 10)     //convert year to two-digit
            cout << 0 << (year % 100);
         else
            cout << (year % 100);
         cout << endl;
      }
   else if (dateFormat == 'L')   //long format
      {
         cout << monthName[month-1] << " " << day << ", " << year << endl;
      }
   else if (dateFormat == 'J')   //Julian format
      {
         for (int i = 1; i < month; i++)
            {
               julian += MonthLen(i,year); //add up all the prev months' days
            }
         julian += day;    //plus up to and including the day specified
         
         if ((year % 100) < 10)     //convert year to two-digit
            cout << 0 << (year % 100);
         else
            cout << (year % 100);
         
         cout << "-";
         
         if (julian < 100)    //leading zero
            cout << 0 << julian;
         else
            cout << julian;
         
         cout << endl;
      }
}

bool Date::Set(int m, int d, int y)
{
   int allGood = 0;
   
   if (m > 0 && m < 13)    // check month
      allGood += 1;      
   if (d > 0 && d <= MonthLen(m,y))     // check day
      allGood += 1;
   if (y >= 0)    //check year
      allGood += 1;
   if (allGood == 3) //if they're all good, set 'em!
      {  
         month = m;
         day = d;
         year = y;
         allGood = 0;
         return true;
      }
   else     //if not, don't do anything, and return false
      return false;
}

void Date::Increment(int numDays)
{
   for (int i = 1; i <= numDays; i++)
      {   
         if (day == MonthLen(month,year))    //if last day of month
            {
               day = 1;
               if (month == 12)     //if last month of year
                  {
                     month = 1;
                     year++;
                  }
               else
                  month++;
            }
         else
            day++;
      }
}

int Date::MonthLen(int m, int y) const
{
   int monthLen[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
   if (y % 4 == 0)   //calculate leap year
      {
         if (y % 100 == 0 && y % 400 != 0)  //if a century but not div by 400
            monthLen[1] = 28;    //its not a leap year
         else
            monthLen[1] = 29;    //otherwise it is
      }
   return monthLen[m-1];
}

bool Date::SetFormat(char f)
{
   if (f == 'D' || f == 'T' || f == 'L' || f == 'J')
      {
         dateFormat = f;
         return true;
      }
   else
      return false;
}

int Date::Compare(const Date& d) const
{
   if (d.GetYear() == year)
      {
         if (d.GetMonth() == month)
            {
               if (d.GetDay() == day)
                  {
                     return 0;
                  }
               else if (d.GetDay() > day)
                  return -1;
               else if (d.GetDay() < day)
                  return 1;
            }
         else if (d.GetMonth() > month)
            return -1;
         else if (d.GetMonth() < month)
            return 1;
      }
   else if (d.GetYear() > year)
      return -1;
   else if (d.GetYear() < year)
      return 1;
}

void Date::Input()
{
   int m,d,y;
   char a,b;   //eat the slashes
   cout << "Please enter a date in the format m/d/yyyy: ";
   cin >> m >> a >> d >> b >> y;
   if (!Set(m,d,y))
      {
         cout << "\nInvalid date entered. Please try again.\n\n";
         Input();
      }
}

int Date::GetMonth() const
{
   return month;
}

int Date::GetDay() const
{
   return day;
}

int Date::GetYear() const
{
   return year;
}
